define(function(require) {
	var $ = require('jquery');
	require("cordova!org.apache.cordova.device");
	var JPushInstance = function() {
		if (window.plugins && window.plugins.jPushPlugin) {

			/*该事件是在PhoneGap载入完成后 发生的事件
			它表示cordova 加载完成并准备访问 . yourCallbackFunction
			相当于程序的入口功能
			如下 onDeviceReady 函数*/

			//自定义 设备准备好 通知
			document.addEventListener("deviceready", this.onDeviceReady.bind(this), false);
			//获取点击通知内容    点击通知进入应用程序时会出发改事件
			document.addEventListener("jpush.openNotification", this.onOpenNotification.bind(this), false);
			//获取通知内容    点击通知进入应用程序时会触发该事件
			document.addEventListener("jpush.receiveNotification", this.onReceiveNotification.bind(this), false);
			//获取自定义消息推送内容   收到应用内消息时触发这个事件
			document.addEventListener("jpush.receiveMessage", this.onReceiveMessage.bind(this), false);

			//函数本身无返回值，但需要注册jpush.setTagsWithAlias事件来监听设置结果
			//调用设置别名函数 返回需要事件监听 处理结果
			document.addEventListener("jpush.setTagsWithAlias", this.onAlias.bind(this), false);
		}
	};

	//hcr 解决第一次获取失败的问题
	JPushInstance.prototype.getRegistrationID = function() {

		//alert("jpush.js getRegistrationID");

		var dtd = $.Deferred();
		if (this.registrationID) {
			dtd.resolve(this.registrationID);
		} else {
			if (window.plugins && window.plugins.jPushPlugin) {
				window.plugins.jPushPlugin.getRegistrationID(function(registrationID) {
					self.registrationID = registrationID;
					if (self.registrationID) {
						//是"已完成"（resolved）,deferred对象立刻调用done()方法指定的回调函数
						dtd.resolve(self.registrationID);
					} else {
						//还存在一个deferred.reject()方法，作用是将dtd对象的执行状态从"未完成"改为"已失败"，从而触发fail()方法。
						dtd.reject();
					}
				});
			} else {
				//还存在一个deferred.reject()方法，作用是将dtd对象的执行状态从"未完成"改为"已失败"，从而触发fail()方法。
				dtd.reject();
			}
		}
		//deferred.promise()方法。它的作用是，在原来的deferred对象上返回另一个deferred对象，后者只开放与改变执行状态无关的方法（比如done()方法和fail()方法），屏蔽与改变执行状态有关的方法（比如resolve()方法和reject()方法），从而使得执行状态不能被改变。
		return dtd.promise();
	};

	//设置别名
	JPushInstance.prototype.addPushAlias = function(alias) {
		if (window.plugins && window.plugins.jPushPlugin) {
			window.plugins.jPushPlugin.setAlias(alias);
		}
	}

	//取消别名
	JPushInstance.prototype.delPushAlias = function() {
		if (window.plugins && window.plugins.jPushPlugin) {
			window.plugins.jPushPlugin.setAlias("");
		}
	}

	//自定义 设备准备好    
	JPushInstance.prototype.onDeviceReady = function() {

		var self = this;

		//alert("self中的registrationID "+self.registrationID);
		//alert("this.registrationID "+this.registrationID);

		//调用此API,用来开启 JPush SDK 提供的推送服务
		window.plugins.jPushPlugin.init();

		//集成了 JPush SDK 的应用程序在第一次成功注册到 JPush 服务器时，JPush 服务器会给客户端返回一个唯一的该设备的标识 - RegistrationID。
		//JPush SDK 会以广播的形式发送 RegistrationID 到应用程序。
		//应用程序可以把此 RegistrationID 保存以自己的应用服务器上，然后就可以根据 RegistrationID 来向设备推送消息或者通知。
		window.plugins.jPushPlugin.getRegistrationID(function(registrationID) {
			self.registrationID = registrationID;
		});


		if (device.platform == "Android") {
			window.plugins.jPushPlugin.setDebugMode(false);
			window.plugins.jPushPlugin.setApplicationIconBadgeNumber(0);
		} else {
			window.plugins.jPushPlugin.setDebugMode(false);
			window.plugins.jPushPlugin.setApplicationIconBadgeNumber(0);
		}

	};

	//获取点击通知内容    点击通知进入应用程序时会出发改事件
	JPushInstance.prototype.onOpenNotification = function(event) {
		var alertContent;
		if (device.platform == "Android") {
			alertContent = window.plugins.jPushPlugin.openNotification.alert;
		} else {
			alertContent = event.aps.alert;
		}

		//本接口直接改变应用本地的角标值. 设置iOS的角标，当设置badge＝0时为清除角标
		window.plugins.jPushPlugin.setApplicationIconBadgeNumber(0);

		//起步 提示框
		justep.Util.hint(alertContent);
	};

	//获取通知内容    点击通知进入应用程序时会触发该事件
	JPushInstance.prototype.onReceiveNotification = function(event) {
		var alertContent;
		if (device.platform == "Android") {
			alertContent = window.plugins.jPushPlugin.receiveNotification.alert;
		} else {
			alertContent = event.aps.alert;
		}
		window.plugins.jPushPlugin.setApplicationIconBadgeNumber(0);

		justep.Util.hint(alertContent);
	};

	//获取自定义消息推送内容  收到应用内消息时触发这个事件
	JPushInstance.prototype.onReceiveMessage = function() {
		var message;
		if (device.platform == "Android") {
			message = window.plugins.jPushPlugin.receiveMessage.message;
		} else {
			message = event.content;
		}
		window.plugins.jPushPlugin.setApplicationIconBadgeNumber(0);

		justep.Util.hint(message);
	};

	//自定义处理 别名注册成功函数
	JPushInstance.prototype.onAlias = function(event) {
		//alert("设置别名 返回结果 result code:" + event.resultCode + " " + "alias:" + event.alias + " ");
	};

	return new JPushInstance();
});